/*-----------------------------------------------------------------------------*
// include
*-----------------------------------------------------------------------------*/
#include "harktool.h"




/*-----------------------------------------------------------------------------*
// extern
*-----------------------------------------------------------------------------*/
extern struct GLOBAL  g;






/*-----------------------------------------------------------------------------*
* 幾何計算から伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfgeo( )
{


    harktool_LogI("harktoolcli_calctfgeo start" );



    if(      strlen( g.argv.mic_file ) == 0 )
    {
        harktool_LogE("argument error　[--mic-list]");
        harktool_help( "calctfgeo" );
        harktool_Exit(EXIT_FAILURE);
    }
    else if( strlen( g.argv.tsp_file ) == 0 ) 
    {
        harktool_LogE("argument error　[--tsp-list]");
        harktool_help( "calctfgeo" );
        harktool_Exit(EXIT_FAILURE);

    }
    else if( strlen( g.argv.output ) == 0 )
    {
        harktool_LogE("argument error　[--output-file]");
        harktool_help( "calctfgeo" );
        harktool_Exit(EXIT_FAILURE);

    }
    else if( g.argv.tf_type != LOC && g.argv.tf_type != SEP )
    {
        harktool_LogE("argument error　[--output-type]");
        harktool_help( "calctfgeo" );
        harktool_Exit(EXIT_FAILURE);
    }




    /*--------------------------------------------------*/
    //伝達関数作成
    /*--------------------------------------------------*/
    harktoolcli_calctfgeoExec( );


    /*--------------------------------------------------*/
    //* end proc
    /*--------------------------------------------------*/
    harkio_Log_printf(harkio_Log_Info, "@ harktoolcli_calctfgeo normal end");

    harktool_Exit(EXIT_SUCCESS);


}//end







/*-----------------------------------------------------------------------------*
* 幾何計算から伝達関数作成処理
*
*
*
*-----------------------------------------------------------------------------*/
void harktoolcli_calctfgeoExec( )
{


    harktool_LogI("harktoolcli_calctfgeoExec start" );


    int src,mic,fft;
    const float sound_speed = 343;



    /*--------------------------------------------------*/
    // micファイル(microphones.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.mic_xml = harkio_XML_newFromXML( g.argv.mic_file , "" );


    /*--------------------------------------------------*/
    // tsp-listファイル(source.xml)->XML構造体へ
    /*--------------------------------------------------*/
    g.tsp_xml = harkio_XML_newFromXML( g.argv.tsp_file , "" );


    /*--------------------------------------------------*/
    // position type
    /*--------------------------------------------------*/
    g.PosType = TSP;


    /*--------------------------------------------------*/
    //mic数
    /*--------------------------------------------------*/
    g.Nmic = harkio_Positions_getSize( g.mic_xml->poses );



    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    g.Nsrc = harkio_Positions_getSize( g.tsp_xml->poses );


    /*--------------------------------------------------*/
    // Config取得
    /*--------------------------------------------------*/
    g.Nfft           = harkio_Config_getNfft( g.tsp_xml->cfg );
    g.TspLength      = harkio_Config_getTSPLength( g.tsp_xml->cfg );
    g.SamplingRate   = harkio_Config_getSamplingRate( g.tsp_xml->cfg );
    g.SignalMax      = harkio_Config_getSignalMax( g.tsp_xml->cfg );
    g.SyncAverage    = harkio_Config_getSynchronousAverage( g.tsp_xml->cfg );
    g.OrgTspPath     = harkio_Config_getTSPpath( g.tsp_xml->cfg );
    g.TspOffset      = harkio_Config_getTSPoffset( g.tsp_xml->cfg );
    g.PeakSearchFrom = harkio_Config_getPeakSearchFrom( g.tsp_xml->cfg );
    g.PeakSearchTo   = harkio_Config_getPeakSearchTo( g.tsp_xml->cfg );

    g.IRsize = g.Nsrc * g.Nmic * g.Nfft ;
    g.TFsize = g.Nsrc * g.Nmic * ( g.Nfft/2 + 1) ;


    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/
    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
    harktool_new( TF_VECT );


    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    harktool_new( TMP_TF_VECT );


    //伝達関数作成用TEMP領域
    harktool_new( BUFFER );


    // mic position格納領域(g.MicPosVect)
    harktool_new( MIC_POS_VECT );

    //SRC position
    harktool_new( SRC_POS_VECT );


    //SORT
     harktool_new( SORT_POS_ID );

    // インパルス応答ファイルのPATH格納領域(tmp.flt_path)
    harktool_new( FLT_PATH );


    // TSP wavファイル名格納領域
    harktool_new( TSP_WAV_NAME );





    /*--------------------------------------------------*/
    /*  Create Directory                                */
    /*--------------------------------------------------*/
    harktool_createTmpPath() ;


    /*--------------------------------------------------*/
    // id を昇順に並べ替えてpositionをセット
    /*--------------------------------------------------*/
    //MIC ID 並べ替え
    harktool_setMicPosId();

    //mic positionセット + 極座標->直交座標変換処理
    harktool_setMicPos( );


    // TSP ID 並べ替え
    harktool_setTspPosId();

    //tsp positionセット + 極座標->直交座標変換処理
    harktool_setSrcPos( );



    /*--------------------------------------------------*/
    // 計算処理
    /*--------------------------------------------------*/

    //mic positionの平均
    harktool_calcMicPosMean( );


//    harktool_LogE( "PosMean[0]=%f",  g.PosMean[0] );
//    harktool_LogE( "PosMean[1]=%f",  g.PosMean[1] );
//    harktool_LogE( "PosMean[2]=%f",  g.PosMean[2] );



    for( mic=0 ; mic< g.Nmic ; mic++)
    {
        g.MicPosVect[mic][0] = g.MicPosVect[mic][0] - g.PosMean[0];
        g.MicPosVect[mic][1] = g.MicPosVect[mic][1] - g.PosMean[1];
        g.MicPosVect[mic][2] = g.MicPosVect[mic][2] - g.PosMean[2];
    }


    float sound_speed_1;
    sound_speed_1 = g.SamplingRate / sound_speed;



    // norm_2 v = sqrt (sum (v [i] * v [i]))


    float src_mic_delta;
    float src_pos[3];
    float square , x_pos , y_pos , z_pos;
    float theta;

    for( src=0 ; src< g.Nsrc ; src++)
    {

        src_pos[0] = g.SrcPosVect[src][0]; //x
        src_pos[1] = g.SrcPosVect[src][1]; //y
        src_pos[2] = g.SrcPosVect[src][2]; //z


        for( mic=0 ; mic< g.Nmic ; mic++)
        {
             x_pos = src_pos[0] - g.MicPosVect[mic][0];
             y_pos = src_pos[1] - g.MicPosVect[mic][1];
             z_pos = src_pos[2] - g.MicPosVect[mic][2];

            square = harktool_pow(x_pos ,2) + harktool_pow(y_pos ,2) + harktool_pow(z_pos ,2);

            src_mic_delta = sound_speed_1 * (- sqrtf(square));


//            printf("@ [%d] : norm_2[%f] : src_mic_delta[%f]\n", mic,sqrtf(square),src_mic_delta);

            for( fft=0 ; fft< g.Nfft/2+1 ; fft++)
            {
                theta = (2 * pi * src_mic_delta * fft) / g.Nfft ;
                g.TFvect[src][mic][fft] =  cos(theta) + sin(theta) * _Complex_I;
            }
        }
    }




    switch(g.argv.tf_type) //tf_type:SEP,LOC,LOC_SEP
    {
    case LOC:
		//定位伝達関数の配列からharkio_Matrix型に変換する
		harktool_setLoctfFromArray();
		break;

    case SEP:
		//定位伝達関数の配列からharkio_Matrix型に変換する
		harktool_setLoctfFromArray();
		break;
	}

    /*--------------------------------------------------*/
    // 伝達関数をZipファイルに出力
    /*--------------------------------------------------*/
    harktool_writeTF();


    /*--------------------------------------------------*/
    //
    /*--------------------------------------------------*/
    createReadMeToZipFile(g.argv.output);



}//main end





